import java.util.Queue;
import java.util.LinkedList;
import java.util.ArrayList;
import java.util.Scanner;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileNotFoundException;

/**
 * Digit Sum.
 *
 * This program computes the first 10 digits of the sum of a
 * series of 100 50-digit numbers.
 *
 * This hard-codes some things, like the fact that we are only
 * affecting two digits over. If it were truly a file of 
 * N numbers, D digits each, and finding first G digits,
 * this would take a little bit more refactoring.
 */
public class Problem013 implements EulerSolution {

	public static final int N = 100; // how many numbers
	public static final int D = 50; // how many digits
	public static final int G = 10; // how many digits of the sum we want to work out

	public String run() { 
		ArrayList<Queue<Integer>> qs = loadFile("p013_input.txt");
		long firstSum = firstDigits(qs,0,0);
		String first = Long.toString(firstSum);
		return first.substring(0,G);
	}




	/** Given a series of 100 50-digit numbers,
	 * compute the first 10 digits of the sum,
	 * and return them as a long.
	 * */
	public static long firstDigits(ArrayList<Queue<Integer>> qs, int depth, long priorSum) {
		// (Note: columns start at 0, or 10^0, in the 1s place.)
		// Nth column has maximum of 9*N.
		// This number has MD = 3 digits, so each column can affect (MD-1) columns to the left.
		// To estimate column G, we need G columns. We can stop at column G+1.
		// but then we need one more column (not sure why, but depth == G+1).
		// This could use with a bit more generalization.

		// This was an off-by-one that had me stuck for a while.
		if(depth==(G+1)) { 
			// Base case:
			// We have removed/popped the first G columns, 
			// so we know the first G digits.
			return priorSum;
		} else {
			// Recursive case:
			// Accumulate sum of first digits
			long sum = 0;
			for(int j=0; j<N; j++) { 
				Queue<Integer> q = qs.get(j);
				sum += (long)(q.remove());
			}

			// Tally up sum so far
			long runningSum = sum + 10*priorSum;
			// pass this value on to the next,
			// and return whatever results
			return firstDigits(qs, depth+1, runningSum);
		}
	}


	/** Load 100 50-digit numbers from the input file. */
	public static ArrayList<Queue<Integer>> loadFile(String fname) { 
		ArrayList<Queue<Integer>> result = new ArrayList<Queue<Integer>>();
		try{ 
			Scanner s = new Scanner(new BufferedReader(new FileReader(fname)));
			int i=0;
			while(s.hasNextLine()) {
				char[] line = s.nextLine().toCharArray();
				Queue<Integer> q = new LinkedList<Integer>();
				for(int j=0; j<line.length; j++) { 
					q.add( Character.getNumericValue(line[j]) );
				}
				result.add(i,q);
				i++;
			}
		} catch(FileNotFoundException e) { 
			e.printStackTrace();
		}
		return result;
	}
}
