import java.util.Collections;
import java.util.Arrays;
import java.util.Set;
import java.util.HashSet;
import java.util.TreeSet;
import java.util.List;
import java.util.ArrayList;

/** Prime Digits Replacements.
 *
 * Find the smallest prime which, 
 * by replacing part of the number 
 * (not necessarily adjacent digits) 
 * with the same digit, is part of an
 * eight prime value family.
 *
 * Eight prime value family means:
 * when you perform the operation 
 * on the specified pattern,
 * the result is a pool of 8 prime numbers.
 *
 * NOTE: This is working, because it searches
 * for 3 digits to replace, not just 2.
 * Hacking the 2-digit version into a 3-digit version
 * was a lot easier than scrapping the entire approach
 * and reworking it from the ground up.
 */

public class Problem051 implements EulerSolution { 
	public static void main(String[] args) {
		Problem051 p = new Problem051();
		System.out.println(p.run());
	}





	public static Set<Long> primes;
	public static Set<Long> solutionCandidates;
	public static final int POOLSIZE = 8;

	/** Solve the problem. */
	public String run() { 
		int M = 6; // trial and error
		return poolFinder(POOLSIZE,M);
	}

	/** Find the candidate numbers XX*X*XX that have a given prime pool size. */
	public static String poolFinder(int targetPoolSize, int M) { 
		primes = Sieve.primeSieveSet((int)Math.pow(10,M));
		solutionCandidates = new TreeSet<Long>();
		char[] mystuff = new char[M];
		for(int i=0; i<mystuff.length-3; i++) {
			for(int j=i+1; j<mystuff.length-2; j++) {
				for(int k=j+1; k<mystuff.length-1; k++) {
					// Make the recursive call to the method 
					explore( mystuff, i, j, k, 0, targetPoolSize );
				}
			}
		}
		return Long.toString( Collections.min(solutionCandidates) );
	}

	/** Recursive method: fill in the candidate number XX**X with digits,
	 * and once it is full, compare the size of the prime pool to targetPoolSize. */
	public static void explore(char[] arr, int i, int j, int k, int depth, int targetPoolSize) { 

		int M = arr.length;
		if (depth>M) { 
			throw new IllegalArgumentException();
		} else if(depth==M) { 
			// Base case
			arr[i] = '*'; 
			arr[j] = '*';
			arr[k] = '*';
			Pool p = primePool(arr,i,j,k);
			if(p.poolSize==targetPoolSize) {
				for(Long thisp : p.primeSequence) { 
					// bingo, we have a (possible) winner
					solutionCandidates.add(thisp);
				}
			}

		} else if(depth==i||depth==j||depth==k) {
			// Skip this digit recursive case
			explore(arr,i,j,k,depth+1,targetPoolSize);
		} else {
			// Handle this digit recursive case
			//
			// Note: 
			// Could also use Character.forDigit(i) 
			// http://docs.oracle.com/javase/7/docs/api/java/lang/Character.html#forDigit(int,%20int)
			//
			//System.out.println("    Recursive case..."); 
			char startchar;
			if(depth==0) startchar = '1'; else startchar = '0';
			for(char c = startchar; c <= '9'; c++) { 
				arr[depth] = c;
				explore(arr,i,j,k,depth+1,targetPoolSize);
			}
		}
	}


	public static String stringifySequence(char[] stuff) { 
		StringBuffer sb = new StringBuffer();
		for(int i=0; i<stuff.length; i++) { 
			sb.append(stuff[i]);
		}
		return sb.toString();
	}


	/** For a number with a missing digit at index i, determine the size 
	 * of the prime number pool when the missing digit is filled with 0-9. */
	public static Pool primePool(char[] arr, int i, int j, int k) {
		// Start with a fresh copy
		char[] fresh = arr.clone();
		// Compute the prime pool
		int primePool = 0;
		// Note that a leading 0 doesn't count, as per the *3 example
		char startchar;
		if(i==0) startchar = '1'; else startchar = '0';

		// Replace i with 0/1 through 9, and check if each is a prime
		ArrayList<Long> primeSequence = new ArrayList<Long>();
		for(char c = startchar; c<='9'; c++){
			fresh[i] = c;
			fresh[j] = c;
			fresh[k] = c;
			Long candidate = Long.parseLong(new String(fresh));
			if(primes.contains(candidate)) { 
				primePool++;
				primeSequence.add(candidate);
			}
		}
		return new Pool(primePool,primeSequence);
	}

	/** Composite object that bundles a pool size with a pool. */
	protected static class Pool {
		public int poolSize;
		public ArrayList<Long> primeSequence;
		public Pool(int s, ArrayList<Long> p) {
			poolSize = s; 
			primeSequence = p;
		}
	}
}


