import java.util.*;
import java.io.*;

/**
 * Largest Product in a Grid.
 *
 * In a 20 x 20 grid, four numbers along the diagonal are marked,
 * whose product is 26 x 63 x 78 x 14 = 1788696
 *
 * What is the greatest product of four adjacent numers in same direction,
 * up, down, left, right, or diagonally, in 20x20 grid?
 */
public class Prob11 {

	final static int N = 20; // size of problem is fixed
	final static int H = 4; // size of adjacent numbers 

	/** Main method.
	 * 1. Load the problem.
	 * 2. Create the stencil.
	 * 3. Iterate over the grid, finding max products.
	 * 4. Report the result.
	 */
	public static void main(String[] args) throws FileNotFoundException {

		// Step 1:
		// Load the problem
		Scanner s = new Scanner(new BufferedReader(new FileReader("input.txt")));
		int[][] grid = new int[N][N];
		int i = 0;
		while(s.hasNextInt()) { 
			int r = i/N;
			int c = i%N;
			grid[r][c] = s.nextInt();
			i++;
		}

		// Step 2:
		// Create the stencil
		int[][] stencil = new int[H][H];

		// Step 3:
		// Iterate over the grid, finding max products.

		int maxProduct = 0;

		// This needs to be (N-H)+1 because 
		// we need to include (N-H) in the stencil. 
		for(int r = 0; r < (N-H)+1; r++) {
			for(int c = 0; c < (N-H)+1; c++) { 

				// Populate the stencil array
				for(int sr=0; sr<H; sr++){
					for(int sc=0; sc<H; sc++) { 
						stencil[sr][sc] = grid[r+sr][c+sc];
					}
				}

				// This is gonna be a bit ugly.
				// Start with row sums: 0 and 3.
				// Then column sums: 0 and 3.
				// Then diagonal sums: +x and -x.
				maxProduct = Math.max( maxProduct, rowProduct(stencil, H, 0) );
				maxProduct = Math.max( maxProduct, rowProduct(stencil, H, 3) );
				maxProduct = Math.max( maxProduct, colProduct(stencil, H, 0) );
				maxProduct = Math.max( maxProduct, colProduct(stencil, H, 3) );
				maxProduct = Math.max( maxProduct, diagProduct(stencil, H, true) );
				maxProduct = Math.max( maxProduct, diagProduct(stencil, H, false) );

			}
		}
		System.out.println("Maximum product found: " + maxProduct);


	}

	public static int diagProduct(int[][] stencil, int H, boolean topLeft) { 
		int prod = 1;
		if(topLeft) { 
			// top left to bottom right diagonal
			for(int i=0; i<H; i++) { 
				prod *= stencil[i][i];
			}
		} else {
			// top right to bottom left diagonal
			for(int i=0; i<H; i++) { 
				prod *= stencil[i][H-(i+1)];
			}
		}
		return prod;
	}


	public static int colProduct(int[][] stencil, int H, int col) { 
		if(col>=H) { 
			throw new IllegalArgumentException("invalid column or size: col = "+col+" H = "+H);
		}
		int prod = 1;
		for(int i=0; i<H; i++) { 
			prod *= stencil[i][col];
		}
		return prod;
	}

	public static int rowProduct(int[][] stencil, int H, int row) { 
		if(row>=H) { 
			throw new IllegalArgumentException("invalid row or size: row = "+row+" H = "+H);
		}
		int prod = 1;
		for(int i=0; i<H; i++) { 
			prod *= stencil[row][i];
		}
		return prod;
	}

	/** Print out a grid. */
	public static void printGrid(int[][] grid) { 
		for(int j=0; j<grid.length; j++) { 
			System.out.println(Arrays.toString(grid[j]));
		}
	}
}
