/**
 * Highly Divisible Triangular Number.
 *
 * This program finds the first triangular number with over 500 divisors.
 */
public class Triangular {
	public static void main(String[] args) { 
		findTriangularNumber(500);
	}

	/** Find a triangular number with over k divisors. */
	public static void findTriangularNumber(long k) { 
		int divisors = 0;

		// Note: the smallest number with 500 factors 
		// has approx log_2( 500 ) prime factors.
		// That's 8.9, or rounding down, 8. 
		// A number with 2^9 prime divisors has 512 factors.
		//
		// Next, we know the value of n is proportional to the 
		// square root of the expression, so we approximate
		// our starting point by picking a number
		// that will start at or below the right neighborhood
		// and work our way up.
		// 
		long n = (long)(Math.sqrt(2*3*5*7*11*13*17*19));


		// We want first number with OVER k divisors
		while(divisors <= k) { 
			n++;
			divisors = countDivisors(triangle(n));
			//System.out.printf("%d\t%d\t%d\n",n,triangle(n),divisors);
		}
		System.out.println("Find the nth triangular number with 500 divisors.");
		System.out.println("n = "+n);
		System.out.println("T(n) = "+triangle(n));
		System.out.println("Number of divisors = "+countDivisors(triangle(n)));
	}

	/** Count the number of divisors of n. */
	public static int countDivisors(long n) { 
		long max = (long)(Math.sqrt(n));
		int divisors = 0;

		// one and itself:
		divisors += 2;
		// unless you're 2
		if(n==2) divisors--;

		// double-count up to sqrt(n)
		for(long i=2; i<=max; i++) { 
			if(gcf(i,n)==i) { 
				// count this divisor, 
				// plus its >sqrt(n) complement
				divisors += 2;
			}
		}
		return divisors;
	}

	/** Perform the Euclidean algorithm to find the greatest common factor between a and b. */
	public static long gcf(long a, long b) {
		long t;
		while(b!=0) { 
			t = b;
			b =  a%b;
			a = t;
		}
		return a;
	}

	/** Return the nth triangular number. */
	public static long triangle(long n) { 
		long tri = (n*(n+1))/2;
		return tri;
	}
}