import java.util.HashMap;

/** Longest Collatz Sequence.
 *
 * The Collatz Sequence is a hailstone algorithm, that is defined by applying
 * the following two rules to n to create the next n in the sequence:
 *
 * n => n/2 (n is even)
 * n => 3n+1 (n is odd)
 *
 * Example sequence:
 * 13 -> 40 -> 20 -> 10 -> 5 -> 16 -> 8 -> 4 -> 2 -> 1
 *
 * What is the starting number under one million that produces
 * the longest chain?
 *
 *
 * Right now, this throws a stack overflow error after 100,000.
 * The sequence length gets to be 350 or so, and then it craps out.
 * We can probably prime the hash table:
 * for the first 1,000 or so, we just add to the hash table.
 * Then we switch to hash table mode.
 */

public class Collatz {
	public static void main(String[] args) { 

		System.out.println("Finding longest Collatz sequence:");

		System.out.printf("n\tlength\n");

		int MAX = 1000000;
		HashMap<Integer,Integer> map = new HashMap<Integer,Integer>();
		int longest = -1;
		int longestN = -1;
		for(int i=1; i<MAX; i++) {

			// For each number, we just want the number of terms until we get to 1.
			// Create a hash table that maps each integer to the number of integers until it gets to 1.
			// As we compute each step of the collatz sequence, we check if the integer is in the hash map.
			// If it is, we use it to finish computing number of chains.
			// If it is not, we continue calculating.
			//
			// Two tricky parts:
			// First is, structuring the loops.
			// Second is, suppose we have the sequence:
			// 13 -> 40 -> 20 -> 10
			// First number is 13.
			// We have not seen 13 before, so we need to add it to hash map.
			// (can only do that once we know the sequence length).
			// Next number is 40. 
			// We have not seen 40 before, so we also need to add it to hash map.
			// (can only do that once we know the sequence length).
			// 
			// Recursive problem?
			// Yes - recursive, memoized problem.
			//
			// (Well... that was a great idea, until I tried it.
			// This craps out at around 350 recursive function instances
			// on the stack, which only gets you to 100,000).

			int length = collatz(i,map);
			if(length>longest) {
				longest = length;
				longestN = i;
				System.out.printf("%d\t%d\n",i,length);
			}
			if(i%50==0) {
				System.out.println(map);
			}
		}
		System.out.println("n = "+longestN);
		System.out.println("Number of terms = "+longest);

	}

	/** Returns number of Collatz sequence terms to get from n to 1. */
	public static int collatz(int n, HashMap<Integer,Integer> hash) { 
		if(n==1) { 
			// Base case: we reached 1
			return 1;

		} else if(hash.containsKey(n)) { 
			// Lucky: already know how many terms it takes to get to 1
			return hash.get(n);

		} else {
			// Recursive case:
			if(n%2==0) { 
				// Even recursive case
				int result = 1 + collatz(even(n), hash);
				hash.put(n,result);
				return result;
			} else {
				// Odd recursive case
				int result = 1 + collatz(odd(n), hash);
				hash.put(n,result);
				return result;
			}
		}
	}

	public static int even(int n) { return n/2; }
	public static int odd(int n) { return 3*n+1; }

}