import java.util.Arrays;

public class Path {

	/** Find the maximum path down a triangle.
	 *
	 * Strategy: add up the marginal sums of each diagonal,
	 * divide by the total number of entries,
	 * and use those measures to make decisions about which 
	 * directions to move in.
	 */
	public static void main(String[] args) {
		System.out.println(navigateMaxPath());
	}


	/** Navigate the triangle one step at a time, making decisions as we go about which direction to move. */
	public static int navigateMaxPath() {
		int[][] triangle = LoadTriangle.loadTriangle();
		int dim = triangle.length;

		int crawl = 0;
		int rr = 0, cc = 0;
		while(rr<dim && cc<dim) { 
			crawl += triangle[rr][cc];

			System.out.print(triangle[rr][cc] + " ");
			if(rr<dim-1 && cc<dim-1) { 
				System.out.println();
				System.out.println("    left sum = " +subtriangleSum(rr+1,cc,triangle) );
				System.out.println("    right sum = "+subtriangleSum(rr+1,cc+1,triangle) );
			}

			if(goLeft(rr,cc,triangle)) { 
				rr++;
			} else {
				rr++; cc++;
			}
		}

		System.out.println();
		return crawl;
	}

	/** Determine whether we should go left at row i, column j of the triangle. */
	public static boolean goLeft(int i, int j, int[][] triangle) { 
		int leftSum  = subtriangleSum( i+1, j, triangle);
		int rightSum = subtriangleSum( i+1, j+1, triangle);
		int dim = triangle.length;
		// break ties by value, but skip last row
		if(leftSum==rightSum && i+1>dim) { 
			return (triangle[i+1][j] > triangle[i+1][j+1]);
		} else {
			return leftSum > rightSum;
		}
	}


	/** Compute the probable sum of the subtriange starting at row r, column c.
	 * This weights each entry of row i 
	 * with an even probability, namely,
	 * 
	 * binomial(i,j)/2^i
	 *
	 * This is the fraction of possible 
	 * descending paths that will pass
	 * through/visit this entry in the triangle.
	 *
	 * This approach is not correct because
	 * it weights each entry of the triangle
	 * with equal probability, thereby
	 * throwing away valuable information
	 * (the value of the number).
	 */
	public static int subtriangleSum(int r, int c, int[][] triangle) { 
		int subSum = 0;
		int dim = triangle.length;

		double num = 1.0;
		double denom = 1.0;
		double multiplier = 1.0;

		// Loop over each row, skip the last one
		for(int i=0; r+i<dim; i++){

			// Sum each term, then divide by 2^i.
			// This ensures everything stays an integer.
			int sum = 0;
			denom = Math.pow(2,i);
			for(int j=0; j<=i; j++) { 
				num = Binomial.binomial(i,j);
				multiplier = num;
				sum += multiplier*triangle[r+i][c+j];
			}
			sum /= denom;

			// Add this row's term to the running subtriangle sum
			subSum += sum;
		}
		return subSum;
	}


}