import java.util.Iterator;
import java.math.BigInteger;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.HashMap;

/**
 * Find the product of the first 500500 primes.
 */

public class BigMultiply { 

	public static void main(String[] args) { 
		solve();
	} 

	public static void solve() {
		int N1 = 100500;
		int N2 = 500500507;
		//int N2fac1 = 13;
		//int N2fac2 = 38500039;

		// mod base
		BigInteger base = BigInteger.valueOf(N2);

		// Make a long list of primes - enough that we won't run out
		List<Long> primesList = Sieve.primeSieve( (int)(N1*Math.log(N1)*Math.log(N1)) );

		// This will store the number of times we have used each prime
		TreeMap<Long,Integer> primeUsage = new TreeMap<Long,Integer>();

		// This will memoize power of prime:
		// each key is the prime, 
		// each value is the prime to its power of occurrence
		HashMap<Long,Long> primePower = new HashMap<Long,Long>();

		BigInteger b = BigInteger.ONE;
		BigInteger prime = BigInteger.ONE;

		Long nextPrime = new Long(0);

		Iterator<Long> primeIter = primesList.iterator();

		int primeIndex = 0;
		for(int i=0; i<N1; i++) {
			// Two options:
			// - Multiply by a prime we have not used before
			// - Multiply by a prime that we have used k times before, k+1 times
			// Both of these will double the number of divisors.
			if(i%1000==0) { 
				System.out.println("On divisor 2^"+i);
			}

			// Get next prime
			nextPrime = primesList.get(primeIndex);

			Long optionA, optionB;
			optionA = nextPrime;
			boolean useOptionA = true;

			for(Long primeSoFar : primeUsage.keySet()) { 
				optionB = primePower.get(primeSoFar);
				//System.out.println("Checking option B: primeSoFar = "+primeSoFar+" current count = "+ (primeUsage.get(primeSoFar)) +" option B = "+optionB);
				if(optionB<optionA) { 
					// Use option B

					// Increment the number of times this prime was used
					Integer value = primeUsage.get(primeSoFar);
					//System.out.println("Incrementing value of prime "+primeSoFar+" by "+ (value+1) +" to "+ ((value+1)+value) );
					value = value + (value+1);
					primeUsage.put(primeSoFar,value);

					// memoize the power we will see next time
					primePower.put(primeSoFar, longpow(primeSoFar, value + (value+1) ) );
					
					// Multiply 
					b = b.multiply( BigInteger.valueOf(optionB) ).mod(base);
					//System.out.println("B: Next factor is "+optionB);
					useOptionA = false;
					break;
				}
			}
			if(useOptionA) { 
				// Add this prime to the map of used primes
				primeUsage.put(optionA, new Integer(1));
				//System.out.println("Initializing value of prime "+optionA+" to 1.");

				// add a memoized version of what the power 
				// will be next time (2)
				primePower.put(optionA, optionA*optionA);

				// Multiply 
				b = b.multiply( BigInteger.valueOf(optionA) ).mod(base);
				//System.out.println("A: Next factor is "+optionA);

				// Look at the next prime
				primeIndex++;
			}
		}
		System.out.println("Result:");
		System.out.println(b);
		System.out.println();
	}

	public static Long longpow(Long base, Integer exponent) {
		Long result = new Long(1);
		for(int i=0; i<exponent; i++) { 
			result *= base;
		}
		return result;
	}
}