import java.util.*;
import java.math.BigInteger;

/**
 * Test PrimeGenerator classes.
 *
 * This tests the following prime number
 * generator classes:
 * - PrimeGeneratorBig
 * - PrimeGeneratorLong
 *   
 */
public class TestPrimes {

	public static void main(String[] args) { 
		//reallyStupidSimple();
		//stupidSimple();
		//justPrimeGenerator();
		//testNest1();
		//testNest2();

		rangeTest();
	}



	/////////////////////////////////////
	// PrimeGeneratorLong


	public static void rangeTest() {
		System.out.println(PrimeGeneratorLong.primesRange(1000000000000L,1000000000100L));
	}



	public static void testNest1() {
		long MAX = (long)(1E6);
		PrimeGeneratorLong pg1 = new PrimeGeneratorLong(MAX/6);
		long p1;
		while(pg1.hasNext()) { 
			p1 = pg1.next();
		}
	}

	public static void testNest2() {

		long MAX = (long)(1E6);

		PrimeGeneratorLong pg1 = new PrimeGeneratorLong(MAX/6);
		PrimeGeneratorLong pg2 = new PrimeGeneratorLong(MAX/6);
		PrimeGeneratorLong pg3 = new PrimeGeneratorLong(MAX/6);

		long p1, p2, p3, product;
		long eightFactors = 0;
		int whatever = 0;

		while(pg1.hasNext()) { 
			p1 = pg1.next();
			if(p1*p1*p1>MAX) { 
				continue;
			}
			pg2.reset();
			while(pg2.hasNext()) { 
				p2 = pg2.next();
				if(p2<=p1 || p1*p2*p2 > MAX) { 
					continue;
				}
				pg3.reset();
				while(pg3.hasNext()){ 
					p3 = pg3.next();
					if(p3<=p2) { 
						continue;
					}

					product = p1*p2*p3;
					whatever++;

					if(product<=MAX){
						eightFactors++;
					} else {
						// This will save our necks.
						break;
					}

				}

			}
		}
	}

	public static void reallyStupidSimple() {
		System.out.println(PrimeGeneratorLong.primesRange(900,1000));
	}

	public static void stupidSimple() { 
		PrimeGeneratorLong pg = new PrimeGeneratorLong(1000);
		while(pg.hasNext()) { 
			System.out.println( pg.next() );
		}

		PrimeGeneratorLong pgX = new PrimeGeneratorLong(7);
		while(pgX.hasNext()) { 
			System.out.println(pgX.next());
		}
	}


	public static void justPrimeGenerator() { 
		PrimeGeneratorLong pg = new PrimeGeneratorLong((long)(1E9));
		while(pg.hasNext()) { 
			Long prime = pg.next();
		}
	}





	/////////////////////////////////////
	// PrimeGeneratorBig

	public static void bigDigit() { 
		PrimeGeneratorBig pg = new PrimeGeneratorBig( new BigInteger("1000000000") );
		while(pg.hasNext()) { 
			BigInteger prime = pg.next();
		}
	}

	public static void timeBigDigit() { 
		long tick;
		long tock;
		double duration;

		PrimeGeneratorBig pg = new PrimeGeneratorBig( new BigInteger("1000000000") );
		long i = 0;
		tick = System.nanoTime();
		while(pg.hasNext()) { 
			i++;
			BigInteger prime = pg.next();
			if(i%10000000==0) { 
				tock = System.nanoTime();
				duration = (tock-tick)/1E9;
				System.out.printf("i = %d\t\tprime = %d\t\tduration = %.3f \n",i,prime,duration);
				tick = System.nanoTime();
			}
		}
	}


	public static void howLongRangeTo1e12() { 
		long tick;
		long tock;
		double duration;
		List<BigInteger> primes;

		System.out.println("Checking how long for primesRange 1e12 to 1e12+1000.");

		tick = System.nanoTime();
		primes = PrimeGeneratorBig.primesRange(new BigInteger("1000000000000"), new BigInteger("1000000001000") );
		tock = System.nanoTime();

		duration = (tock-tick)/1E9;
		System.out.printf("Took %.3f seconds for primesRange to check for 1000 primes around 1E12.\n",duration);
	}


	public static void testPrimeGeneratorBig() { 
		PrimeGeneratorBig pg = new PrimeGeneratorBig( new BigInteger("100000") );
		int i = 0;
		while(pg.hasNext()) { 
			i++;
			BigInteger prime = pg.next();
			if(i%1000==0) { 
				System.out.println(prime);
			}
		}
	}

	public static void testSegmentedSieve() { 
		List<BigInteger> primes;
		
		primes = PrimeGeneratorBig.primesRange(new BigInteger("100"), new BigInteger("200") );
		System.out.println(primes);

		primes = PrimeGeneratorBig.primesRange(new BigInteger("10000000000"), new BigInteger("10000000100") );
		System.out.println(primes);
	}

}
