from pprint import pprint

"""
Castle Variations

Given a convex castle, find runs of consecutive Rs
and construct all variations of this castle.



This does not account for the fact that if we have
multiple runs, variations of each run actually create
new castles.

That is, if we have 13 variations from the first set 
of consecutive Rs, plus the original convex castle, 
and 15 variations from the second set, plus original,
we actually have (13 + 1) x (15 + 1) = 224 variations.

If we have n slots, and each slot j has V_j variations,
the total number of variations on that one single
convex castle is:

    \Pi_{j=1}^{n} (V_j + 1)



TODO:

The script is currently able to correctly create
variations for heights equal to max height H.

It does not correctly create variations 
if the height is less than H.

Bingo - the problem is with the variant construction.
Take the example:

    U U R R R R U U R D D 

This run of 4 Rs, and max height of 4,
has no constructed variations of the form

    R R D R U R

(None starting with two Rs)

The constructed variations are missing these,
and that leads to the 
"""


def main():
    #working_verification()
    broken_verification()


def working_verification():
    """
    Construct castle verifications and verify 
    a formula for the number of variations.
    """
    # Verification successful for these cases.
    # The runs of Rs are at the maximum height H.
    construct_castle_variations("UUUURRRDDDD")
    construct_castle_variations("UUUURRRRDDDD")
    construct_castle_variations("UUUURRRRRDDDD")
    construct_castle_variations("UUUURRRRRRDDDD")
    construct_castle_variations("UUUURRRRRRRDDDD")
    construct_castle_variations("UUUURRRRRRRRDDDD")
    construct_castle_variations("UUUURRRRRRRRRDDDD")

    # Verification successful for runs of Rs
    # that are at height 1.
    construct_castle_variations("URRRRUUURDDDD")
    construct_castle_variations("URRRRRUUURDDDD")
    construct_castle_variations("URRRRRRUUURDDDD")
    construct_castle_variations("URRRRRRRUUURDDDD")
    construct_castle_variations("URRRRRRRRUUURDDDD")
    construct_castle_variations("URRRRRRRRRUUURDDDD")
    construct_castle_variations("URRRRRRRRRRUUURDDDD")

    # Verification results in NO runs
    construct_castle_variations("URRURRURRDDD")


def broken_verification():
    """
    Construct castle verifications and verify 
    a formula for the number of variations.
    """
    # Verification incorrect for runs lower than height H

    ## 3 Rs = correct (but only 1 run)
    #construct_castle_variations("UURRRUURDDD")
    #construct_castle_variations("UURRRUUURDDDD")
    #construct_castle_variations("UURRRUUUURDDDDD")
    #construct_castle_variations("UURRRUUUUURDDDDDD")
    
    # 4 Rs = off by one
    # FIX THIS FIRST
    construct_castle_variations("UURRRRUURDDD")
    #construct_castle_variations("UURRRRUUURDDDD")
    #construct_castle_variations("UURRRRUUUURDDDDD")
    #construct_castle_variations("UURRRRUUUUURDDDDDD")

    # 5 Rs = off by:
    #    H = 4:     off by 5
    #    H = 5:     off by 6
    #    H = 6:     off by 7
    #construct_castle_variations("UURRRRRUURDDD")
    #construct_castle_variations("UURRRRRUUURDDDD")
    #construct_castle_variations("UURRRRRUUUURDDDDD")
    #construct_castle_variations("UURRRRRUUUUURDDDDDD")



    # Verification only correct for third run
    #construct_castle_variations("UUUURRRRUUUURRRRUUUURRRRDDDDDDDDRRRRDDDD")


def test_first_nonzero():
    """
    Test the first nonzero number
    """

    print("These should be True:")
    print(first_nonzero_number_negative([2,2,1,7]))
    print(first_nonzero_number_negative([2,2,1,2]))
    print(first_nonzero_number_negative([2,1,7,2]))

    print("These should be False:")
    print(first_nonzero_number_negative([2,3,4,2]))
    print(first_nonzero_number_negative([2,7,2,1]))
    print(first_nonzero_number_negative([2,7,6,1]))
    print(first_nonzero_number_negative([2,2,2,2]))


def construct_castle_variations(castle):
    """
    Find the index, length, and height 
    of all seqences of Rs.
    """
    # Find index, length, and height of R sequences
    runs = consecutive_Rs(castle)

    H = sum([1 for m in castle if m=='U'])

    variant_sizes = []
    for run_ix, run_len, run_h in runs:
        if(run_len<3):
            # Not enough Rs
            continue

        castles = []
        heights = [run_h]
        ix = run_ix
        p = run_len - 2
        variants(castle, castles, ix, 0, p, heights, H)

        variant_sizes.append(len(castles))

        debug("For variant %s: %d runs"%((run_ix,run_len,run_h),len(castles)))

        # Don't include original castle... (change to 1 to include it)
        verif = 0
        for k in range(1,p+1):

            # This is solid when run of Rs is at height H
            verif += (run_h - 1)*pow(H,k-1)

        debug("Verification value: %d"%(verif))
        for c in castles:
            debug(c)
        debug("")

    result = prod([j+1 for j in variant_sizes])
    debug("Total castle variations counted: %s"%(result), channel="shut up")


def consecutive_Rs(castle):
    """
    Find sequences of multiple Rs.
    Return a tuple of pairs (x,y),
    where x is the starting index and 
    y is the length of the run.
    """
    runs = []
    doing_run = False
    current_length = 0
    current_index = 0
    current_height = 0
    for im,move in enumerate(castle):

        if(doing_run):
            if(move is not 'R'):
                # If not R and doing run,
                # end the current run
                runs.append((current_index, current_length, current_height))
                doing_run = False
                current_length = 0
                current_index = 0
                current_height = 0
            else:
                # If R and doing a run,
                # increment length
                current_length += 1
        else:
            if(move is 'R'):
                # If R and not doing a run,
                # find the height and
                # start doing a run
                doing_run = True
                current_index = im
                current_length = 1
                for k in range(im):
                    if castle[k] is 'U':
                        current_height += 1
                    elif castle[k] is 'D':
                        current_height -= 1

    return runs


def variants(castle, castles, ix, which_p, max_p, heights, max_height):
    """
    Construct variations of castle 
    (which is a convex castle).

    NOTE: 
    If we are just passing in sequences of R's,
    and we have the same set of heights reoccurring,
    this function can be memoized.

    NOTE 2:
    This DOES NOT include/generate the original convex castle.

    castle is the string of moves representing 
    this convex castle.

    castles is a container storing all castle 
    variations.

    which_p is the current column. 
    If we have m consecutive R's,
    we have p = m-2 columns to set.
    First and last column heights are fixed.

    max_p is the (max) number of columns whose height
    can be set (run length - 2).

    heights is a list of m-1 heights.
    The very first height is the starting height.
    The very last height is the same (and is not added).
    Implicitly, the very last R is not touched.
    If it were a different height, the variant would be a dupe.

    max_height is the maximum height this castle
    can achieve (H).
    """
    if(which_p==max_p):

        all_elements_the_same = True
        for i in range(1,len(heights)):
            if(heights[i]!=heights[0]):
                all_elements_the_same = False

        if(all_elements_the_same):
            # All heights are the same
            # This is the original convex castle
            return

        else:
            # base case:
            # Use heights to assemble correct insertions of Us and Ds.
            # 
            # Example:
            #   RRRR
            # We have four columns, but only two with a variable height.
            # 
            # Insert first R (at height h). Change to new height.
            # 
            # Insert second R (at new height). Change to new new height.
            # 
            # Insert third R (at new new height). (No new height to set - return to original height.)
            # 
            # Insert last R.

            left_piece = castle[0:ix]
            right_piece = castle[ix+max_p+2:]

            # Last height equals first height
            heights.append(heights[0])

            buff = []

            buff.append('R')

            for i in range(1,len(heights)):
                dh = heights[i] - heights[i-1]
                if(dh>0):
                    buff.append('U'*dh)
                elif(dh<0):
                    buff.append('D'*(-dh))

                buff.append('R')

            final_castle = left_piece + "".join(buff) + right_piece

            castles.append(final_castle)

            debug(heights)

            heights.pop()

    else:
        # recursive case:
        for this_h in range(1,max_height+1):
            heights.append(this_h)

            if(first_nonzero_number_negative(heights)):
                variants(castle, castles, ix, which_p+1, max_p, heights, max_height)

            heights.pop()


def prod(arr):
    """
    Take the product of every element in an array.
    (Why does Python provide sum, but not prod?)
    """
    prod = 1
    for a in arr:
        prod *= a
    return prod


def debug(text, channel=None):
    if channel is None:
        print(text)


def diff(vec):
    dvec = []
    for i in range(1,len(vec)):
        dvec.append(vec[i]-vec[i-1])
    return dvec


def first_nonzero_number_negative(vec):
    """
    Boolean function: is the first nonzero 
    change in height negative or positive?
    """
    dvec = diff(vec)
    for d in dvec:
        if(d<0):
            return True
        elif(d>0):
            return False
        else:
            continue

    # should not be here...
    return False


if __name__=="__main__":
    main()
    #test_first_nonzero()

