from convex_castles import construct_convex_castles, insert_r
import numpy as np

"""
Verification

Verify the formula to count convex castles.
"""



def main():
    test_convex_count()
    #test_consecutive_count()
    #test_all_consecutive_count()


########################################################
# Testing formulas for counting 
# number of convex castles.


def test_convex_count():
    """
    Call the function that verifies our 
    convex castle counting formula.
    """
    pairs = [(4,2),
             (2,4),
             (3,5),
             (5,3),
             (2,5),
             (5,2),
             (7,4),
             (4,7)]

    for W,H in pairs:
        verify_convex_count(W,H)


def verify_convex_count(W,H):
    """
    Verify the count of convex castles 
    on a W x H grid.

    This uses construct_convex_castles() to
    actually construct each convex castle,
    then uses a formula to count the convex
    castles:

        \binom{  W - 1 + 2 (H-1)  }{  2 (H - 1) }
    """
    convex_castles = construct_convex_castles(W, H)
    castle_count = count_convex_castles(W,H)

    print("Verification results (numbers should match):")
    print("Count from construct_convex_castles(%d, %d):\t%d"%(W, H, len(convex_castles)))
    print("Count from count_convex_castles(%d, %d):    \t%d"%(W, H, castle_count))
    print()


def count_convex_castles(W,H):
    """
    Returns the count of convex castles
    on a W x H grid:

        \binom{  W - 1 + 2 (H - 1)  }{  2 (H - 1) }

    """
    result = 1

    k = 2*(H-1)
    n = (W-1)+k

    # Return binon{n}{k}

    num = fact(n)
    denom = fact(k)*fact(n-k)

    return num/denom




########################################################
# Testing formulas for counting 
# number of castles with M consecutive Rs.
# 
# NOTE: This approach has been abandoned.

### def test_all_consecutive_count():
### 
###     #          W,H,m,p
###     dparams = [(4,2,2),
###                (4,2,3)]
### 
###     for W,H,m in dparams:
###         verify_all_consecutive_count(W,H,m)
### 
### 
### def verify_all_consecutive_count(W,H,m):
###     """
###     Verifies the formula for counting how many 
###     convex castles have m consecutive Rs in any slots.
### 
###     This uses construct_convex_castles() 
###     to construct all possible convex castles,
###     then counts the number of castles with 
###     m consecutive Rs in any slot.
###     """
### 
###     convex_castles = construct_convex_castles(W, H)
### 
###     valid = 0
###     for castle in convex_castles:
###         u_locs = [i for i,c in enumerate(list(castle)) if c=='U']
###         d_locs = [i for i,c in enumerate(list(castle)) if c=='D']
### 
###         locs = u_locs + d_locs
###         dlocs = [int(j-1) for j in np.diff(locs)]
### 
###         if m in dlocs:
###             valid += 1
### 
###     # Step through all slots
###     all_consecutive_count = 0
###     all_consecutive_count = count_all_consecutive(W, H, m)
### 
###     print("Count of all from constructed castles (%d, %d, %d):       \t%d"%(W, H, m, valid))
###     print("Count of all from consecutive count formula (%d, %d, %d): \t%d"%(W, H, m, all_consecutive_count))
### 
### 
### 
### def count_all_consecutive(W, H, m):
###     """
###     Use the formula to count convex castles
###     with exactly M Rs in any slot.
###     This accounts for the middle,
###     which is why it is correct, and repeated
###     calls to count_consecutive() are not
###     """
###     k1 = 2*(H-1)-1
###     n1 = W-1-m+k1
###     num1 = fact(n1)
###     denom1 = fact(k1)*fact(n1-k1)
###     term1 = 2*(H-1)*(num1/denom1)
### 
###     k2 = k1
###     n2 = W-m+k2
###     num2 = fact(n2)
###     denom2 = fact(k2)*fact(n2-k2)
###     term2 = num2/denom2
### 
###     return term1 + term2
### 
### 
### 
### 
### def test_consecutive_count():
###     """
###     Call the function that verifies our 
###     consecutive Rs counting formula.
###     """
###     #          W,H,m,p
###     uparams = [(4,4,3,0),
###                (4,4,3,1),
###                (4,4,3,2),
###                (7,3,3,0),
###                (7,3,3,1),
###                (6,4,2,0),
###                (6,4,2,1),
###                (6,4,2,2),
###                (5,6,3,0),
###                (5,6,3,1),
###                (5,6,3,2),
###                (5,6,3,3),
###                (5,6,3,4)]
### 
###     for W,H,m,k in uparams:
###         verify_consecutive_count(W,H,m,k)
### 
###     #          W,H,m,p
###     dparams = [(4,4,3,4),
###                (4,4,3,5),
###                (4,4,3,6),
###                (7,3,3,3),
###                (7,3,3,4),
###                (6,4,2,4),
###                (6,4,2,5),
###                (5,6,3,6),
###                (5,6,3,7),
###                (5,6,3,8),
###                (5,6,3,9)]
### 
###     for W,H,m,k in dparams:
###         verify_consecutive_count(W,H,m,k)
### 
### 
### 
### 
### 
### def verify_consecutive_count(W,H,m,p):
###     """
###     Verifies the formula for counting how many 
###     convex castles have m consecutive Rs in slot p.
### 
###     This uses construct_convex_castles() 
###     to construct all possible convex castles,
###     then counts the number of castles with 
###     m consecutive Rs in slot k.
### 
###     p is indexed starting at 0.
###     """
### 
###     convex_castles = construct_convex_castles(W, H)
### 
###     valid = 0
###     for castle in convex_castles:
###         u_locs = [i for i,c in enumerate(list(castle)) if c=='U']
###         d_locs = [i for i,c in enumerate(list(castle)) if c=='D']
### 
###         # should be len(ulocs)-1,
###         # because we are indexing starting at 0
###         if(p==len(u_locs)-1):
###             raise Exception("Error: you are requesting the middle slot, fix the formula first")
###         
###         elif(p >= 2*(H-1)+1 ):
###             raise Exception("Error: you are requesting too large a slot")
### 
###         elif(p<len(u_locs)):
###             start_index = u_locs[p]+1
###             sec = castle[start_index:start_index+m]
###             if(list(sec) == ['R']*m):
###                 if(castle[start_index+m]=='U'):
###                     valid += 1
###         else:
###             # H prior slots: H-1 UU slots and middle slot
###             start_index = d_locs[p-H]+1
###             sec = castle[start_index:start_index+m]
###             if(list(sec) == ['R']*m):
###                 if(castle[start_index+m]=='D'):
###                     valid += 1
###     
###     consecutive_count = count_consecutive(W, H, m, p)
### 
###     print("Count from constructed castles (%d, %d, %d, %d):       \t%d"%(W, H, m, p, valid))
###     print("Count from consecutive count formula (%d, %d, %d, %d): \t%d"%(W, H, m, p, consecutive_count))
### 
### 
### 
### 
### def count_consecutive(W, H, m, p):
###     """
###     Use the formula to count convex castles
###     with exactly M Rs in slot p
###     assuming p is not the middle slot.
###     """
###     k = 2*(H-1)-1
###     n = W-1-m + k
### 
###     num = fact(n)
###     denom = fact(k)*fact(n-k)
### 
###     return num/denom




def fact(n):
    if(n<=1):
        return 1
    else:
        return n*fact(n-1)
    

if __name__=="__main__":
    main()

