import numpy as np

"""
Project Euler
Problem 181

Count ways of grouping N items of class A, M items of class B.

Example:

    (BBBW)      (B,BBW)     (BB,BW)     (BBB,W)
    (B,B,BW)    (BB,B,W)    (B,B,B,W)

"""

def main():
    #test_cut_string()
    gen_all_groups_debug([4,3])



def test_cut_string():
    """
    Test the cut_string() function.
    """
    result = cut_string("asdf", [1,0,0])
    print(result)

    result = cut_string("aaaaabbbbb", [1,0,0,0,0,1,1,0,0])
    print(result)



def gen_all_groups_debug(class_counts):
    """
    Generate all grouings of objects 
    whose counts are contained in class_counts,
    and record lots of debug information
    to help narrow down what binary numbers 
    (what cut patterns) wind up giving us 
    duplicates.

    From there we can figure out how to 
    count duplicates, and then we can figure
    out how to change the count of ALL possible
    cuts (2^n) to the count of all possible 
    cuts that lead to unique groupings.
    """
    alpha = "abcdefghijklmnopqrstuvwxyz"

    nclasses = len(class_counts)
    ncuts = sum(class_counts) - 1

    original_string = "".join([alpha[i]*j for i,j in enumerate(class_counts)])

    all_groups_list = []
    all_cuts = {}
    for cut in algorithm_m(ncuts, nclasses):
        # cut is a binary number - 
        # a list of integers, 0 or 1, of length 

        cutt = "".join([str(z) for z in cut])

        g = cut_string(original_string,cut)
        all_groups_list.append(g)
        if(g in all_cuts.keys()):
            all_cuts[g].append(cutt)
        else:
            all_cuts[g] = [cutt]

    all_groups_set = set(all_groups_list)
    print(all_groups_set)
    print("Found %d sets total"%(len(all_groups_set)))
    print("Found %d sets including dupliates"%(len(all_groups_list)))
    print()
    for key in all_cuts.keys():
        if( len(all_cuts[key]) > 2 ):
            print("%s\n%s\n\n"%( key, "\n".join(all_cuts[key]) ))

    return all_groups_set



def gen_all_groups(class_counts):
    """
    Generate all (unique) groupings of len(class_counts) objects
    whose counts are given by class_counts.
    """
    alpha = "abcdefghijklmnopqrstuvwxyz"

    nclasses = len(class_counts)
    ncuts = sum(class_counts) - 1

    original_string = "".join([alpha[i]*j for i,j in enumerate(class_counts)])

    all_groups_list = []
    for cut in algorithm_m(ncuts, nclasses):
        # cut is a binary number - 
        # a list of integers, 0 or 1, of length 

        g = cut_string(original_string,cut)
        all_groups_list.append(g)

    all_groups_set = set(all_groups_list)
    print("Found %d sets total"%(len(all_groups_set)))
    print("Found %d sets including dupliates"%(len(all_groups_list)))

    return all_groups_set



def algorithm_m(length, classes):
    """
    Knuth's Algorithm M for permutation generation,
    via AOCP Volume 4 Fascile 2.
    This is a generator that returns permtuations.
    This generates ALL permutations.
    """
    n = length
    # M1 - Initialize
    a = np.zeros(n,)
    m = np.ones(n,)*classes

    j = n-1

    nvisits = 0
    while True:

        # M2 - visit
        move_sequence = [int(aj) for aj in a]
        yield move_sequence 

        nvisits += 1

        # M3 - prepare to +1
        j = n-1

        # M4 - carry
        while( a[j] == m[j]-1):
            a[j] = 0
            j = j-1
        
        # M5 - increase unless done
        if(j<0):
            break
        else:
            a[j] = a[j] + 1



def cut_string( string_to_cut, cut_binary ):
    """
    string_to_cut is a string
    cut_binary is a list of 0s and 1s indicating cut positions
    """
    if(len(string_to_cut) != len(cut_binary)+1):
        raise Exception("Error: asking to cut string of length %d with cut locations of length %d"%(len(string_to_cut),len(cut_locations)))

    cut_locations = [i+1 for i,j in enumerate(cut_binary) if j==1]

    result = []
    if(len(cut_locations)>0): 

        # We have at least one place to cut.

        # fencepost
        result.append(string_to_cut[0:cut_locations[0]])

        for i in range(1,len(cut_locations)):
            result.append( string_to_cut[cut_locations[i-1]:cut_locations[i]] )

        result.append( string_to_cut[cut_locations[-1]:] )
        
        result = sorted(result)

        # Rather than return the list itself, 
        # which is not a hashable type,
        # return a string representation.
        return ",".join(result)

    else:
        # Not cutting anywhere
        return string_to_cut




if __name__=="__main__":
    main()

