def calculate_T(n):
    """
    Calculate T(n), the sum of all 2025-numbers with n digits or less.
    
    A 2025-number is a number 'ab' (concatenation of a and b) where ab = (a + b)²
    """
    # We'll use a set to avoid duplicates
    found_numbers = set()
    
    # Instead of checking all numbers, we generate 2025-numbers directly
    # We know that if (a+b)² = ab, then a+b must be small enough for the square to have ≤ n digits
    
    # The maximum sum (a+b) that could produce a square with n or fewer digits
    max_sum = int(10 ** (n / 2)) + 1
    
    for sum_value in range(1, max_sum):
        square = sum_value ** 2
        
        # If the square has more than n digits, we can stop
        if len(str(square)) > n:
            break
        
        # Check all possible ways to split the sum_value into a and b
        for a in range(1, sum_value):
            b = sum_value - a
            
            # Calculate the concatenation of a and b
            concat_str = str(a) + str(b)
            
            # Check if the concatenation equals the square
            if int(concat_str) == square:
                found_numbers.add(square)
    
    # Sort the results for better readability
    result_list = sorted(found_numbers)
    
    print(f"Found {len(result_list)} 2025-numbers with {n} digits or less:")
    for num in result_list:
        # Find the pair (a,b) that makes this a 2025-number
        num_str = str(num)
        for i in range(1, len(num_str)):
            a = int(num_str[:i])
            b = int(num_str[i:])
            if (a + b)**2 == num:
                print(f"{num} = concat({a}, {b}) = ({a} + {b})² = {a + b}²")
                break
    
    return sum(result_list)

def main():
    import sys
    import time
    
    start_time = time.time()
    
    if len(sys.argv) > 1:
        n = int(sys.argv[1])
        result = calculate_T(n)
        print(f"T({n}) = {result}")
    else:
        # Verify that T(4) = 5131
        n = 4
        result = calculate_T(n)
        print(f"T({n}) = {result}")
        if result == 5131:
            print("Verification successful: T(4) = 5131")
        else:
            print(f"Verification failed: Expected T(4) = 5131, got {result}")
    
    end_time = time.time()
    print(f"Execution time: {end_time - start_time:.6f} seconds")

if __name__ == "__main__":
    main()
